import * as React from 'react';
import * as SelectControlBase from '@tesseract-design/web-base-selectcontrol';
import {RenderOptions} from '@tesseract-design/web-option-react';
import clsx from 'clsx';
import styles from './style.module.css';

export type SliderOrientation = 'horizontal' | 'vertical';

type SliderDerivedElement = HTMLInputElement;

export interface SliderProps extends Omit<React.HTMLProps<HTMLInputElement>, 'type'> {
  orient?: SliderOrientation;
  children?: React.ReactNode;
  length?: React.CSSProperties['width'];
}

export const Slider = React.forwardRef<SliderDerivedElement, SliderProps>(({
  className,
  style,
  children,
  orient = 'horizontal',
  length,
  min = 0,
  max = 100,
  ...etcProps
}, forwardedRef) => {
  const [browser, setBrowser] = React.useState<string>();
  React.useEffect(() => {
    const isFirefox = typeof (window as unknown as Record<string, unknown>).InstallTrigger !== 'undefined';
    if (isFirefox) {
      setBrowser('firefox');
    } else {
      // TODO - detect other browsers
      setBrowser('chrome');
    }
  }, []);

  const defaultRef = React.useRef<HTMLInputElement>(null);
  const ref = forwardedRef ?? defaultRef;
  const tickMarkId = React.useId();

  React.useEffect(() => {
    if (!(typeof ref === 'object' && ref)) {
      return;
    }
    const {current: slider} = ref;
    if (!slider) {
      return;
    }

    const isFirefox = browser === 'firefox';
    const wrapper = slider?.parentElement as HTMLElement;
    const parent = wrapper?.parentElement as HTMLElement;
    const grandParent = parent?.parentElement as HTMLElement;
    if (isFirefox) {
      slider.setAttribute('orient', orient);
      wrapper.dataset[browser] = orient;
      wrapper.removeAttribute('data-orient');
      grandParent.style.width = '0px';
    }

    return () => {
      if (slider && isFirefox) {
        grandParent.style.width = 'auto';
        wrapper.removeAttribute(`data-${browser}`);
        wrapper.dataset.orient = slider.getAttribute(orient) ?? undefined;
        slider.removeAttribute('orient');
      }
    };
  }, [ref, orient, browser]);

  React.useEffect(() => {
    if (!(typeof ref === 'object' && ref)) {
      return;
    }
    const {current: slider} = ref;
    if (!slider) {
      return;
    }

    const wrapper = slider?.parentElement as HTMLElement;
    const parent = wrapper?.parentElement as HTMLElement;
    const grandParent = parent?.parentElement as HTMLElement;

    const isNotFirefox = typeof browser === 'string' && browser !== 'firefox';
    if (isNotFirefox) {
      wrapper.dataset[browser] = orient;
    }

    const shouldEffectExecute = isNotFirefox && orient === 'vertical' && slider && parent && grandParent;
    if (shouldEffectExecute) {
      const trueHeight = parent.clientWidth;
      const trueWidth = parent.clientHeight;

      grandParent.dataset.height = grandParent.clientHeight.toString();
      grandParent.dataset.width = grandParent.clientWidth.toString();
      grandParent.style.height = trueHeight + 'px';
      grandParent.style.width = trueWidth + 'px';
    }

    return () => {
      if (shouldEffectExecute) {
        grandParent.style.height = grandParent.dataset.height + 'px';
        grandParent.style.width = grandParent.dataset.width + 'px';
        grandParent.removeAttribute('data-height');
        grandParent.removeAttribute('data-width');
      }
      wrapper.removeAttribute(`data-${browser}`);
    };
  }, [ref, orient, browser]);

  React.useEffect(() => {
    if (!(typeof ref === 'object' && ref)) {
      return;
    }
    const {current: slider} = ref;
    if (!slider) {
      return;
    }

    const isFirefox = browser === 'firefox';
    const isNotFirefox = typeof browser === 'string' && browser !== 'firefox';
    const tickMarkContainer = slider.nextElementSibling;
    if (tickMarkContainer) {
      const tickMarks = tickMarkContainer.children[0].children;
      Array.from(tickMarks).forEach((tickMarkRaw) => {
        const tickMark = tickMarkRaw as HTMLElement;
        const offset = tickMark.dataset.offset as string;
        if (isNotFirefox) {
          tickMark.style.left = offset;
          tickMark.style.bottom = '';
        } else if (isFirefox && orient === 'horizontal') {
          tickMark.style.left = offset;
          tickMark.style.bottom = '';
        } else {
          tickMark.style.bottom = offset;
          tickMark.style.left = '';
        }
      });
    }
  }, [ref, orient, browser]);

  const block = typeof length === 'string' && length.trim() === '100%';

  return (
    <>
      {
        children
        && (
          <datalist
            id={tickMarkId}
          >
            {children}
          </datalist>
        )
      }
      <div
        className={clsx(
          block && orient !== 'vertical' && 'w-full',
          !block && 'align-middle',
          className,
          orient !== 'vertical' && 'inline-block min-h-[1rem]',
          orient === 'vertical' && 'inline-block min-w-[1rem]',
          orient !== 'vertical' && !block && 'min-w-[16rem]',
          //orient === 'vertical' && 'inline-block min-h-[16rem] min-w-[1rem]',
        )}
        style={style}
      >
        <div
          style={{
            width: orient === 'vertical' ? (length ?? '16rem') : undefined,
            position: 'relative',
          }}
        >
          <div
            className={clsx(
              styles['slider-wrapper'],
              'flex'
            )}
            data-orient={orient}
          >
            <input
              {...etcProps}
              ref={ref}
              min={min}
              max={max}
              type="range"
              list={children ? tickMarkId : undefined}
              className={clsx(
                styles.slider,
                'w-full h-full bg-inherit block text-primary ring-secondary/50 rounded-full',
                'focus:text-secondary focus:outline-0 focus:ring-4',
                'active:text-tertiary active:ring-tertiary/50',
              )}
            />
            {Array.isArray(children) && (
              <div className={clsx(
                styles['tick-mark-container'],
              )}>
                <div className="relative w-full h-full">
                  {children.map((c) => (
                    <div
                      key={c.props.value}
                      className={clsx(
                        styles['tick-mark'],
                        'absolute w-full h-full box-border',
                      )}
                      data-offset={`${(Number(c.props.value) - Number(min)) / (Number(max) - Number(min)) * 100}%`}
                    >
                      {/* TODO fix translateX */}
                      <div
                        className="flex flex-col text-xs items-center justify-between translate-x-[-50%] h-full"
                      >
                        <div
                          className={clsx(
                            'w-[1px] h-2',
                            'bg-current',
                          )}
                        />
                        {c.props.children}
                      </div>
                    </div>
                  ))}
                </div>
              </div>
            )}
          </div>
        </div>
      </div>
    </>
  )
});

Slider.displayName = 'Slider';
